require 'active_support/inflector'

require 'marble_gate/errors'

module MarbleGate
  def self.access(context, action, *args)
    Access.new(context, (action == 'new') ? 'action_new' : action, *args)
  end

  class Access
    attr_reader :context

    def initialize(context, action, *args)
      @context, @action = context, action.to_sym
      @args = args
    end

    def for_model?(model)
      return for_authorizer?(model) if model.is_a?(Class)

      obj = auth_class(model.class).new(model)
      obj.send(choose_action(obj), *@args)
    end

    def for_authorizer?(model_class)
      cls = auth_class(model_class)
      cls.send(choose_action(cls), *@args)
    end

    def user
      context.send(MarbleGate.current_user_method_name)
    end

    private

    def auth_class(model_class)
      return @auth_class if @auth_class

      _auth_class = "#{model_class.name}Authorizer"
      @auth_class = _auth_class.safe_constantize
      raise MarbleGate::MissingAuthorizer unless @auth_class

      %i(user context).each { |m| @auth_class.send("#{m}=", send(m)) }
      @auth_class.model_class = model_class
      @auth_class
    end

    def choose_action(auth)
      return @action if auth.respond_to?(@action)

      new_action = MarbleGate.alias_map[@action]
      return new_action if new_action && auth.respond_to?(new_action)

      return :manage if auth.respond_to?(:manage)

      :default
    end
  end
end
